@@ -30,11 +30,12 @@ use File::Spec;
 use base 'Exporter';
 
 $AUTHORITY = 'SUKRIA';
-$VERSION   = '1.1806_01';
+$VERSION   = '1.1806_02';
 @EXPORT    = qw(
-  ajax
+  after
   any
   before
+  before_template
   cookies
   config
   content_type
@@ -93,10 +94,11 @@ $VERSION   = '1.1806_01';
 
 # Dancer's syntax
 
-sub ajax         { Dancer::Route->add_ajax(@_) }
-sub any          { Dancer::Route->add_any(@_) }
-sub before       { Dancer::Route->before_filter(@_) }
-sub captures   { Dancer::SharedData->request->params->{captures} }
+sub after        { Dancer::Route::Registry->hook('after', @_) }
+sub any          { Dancer::App->current->registry->any_add(@_) }
+sub before       { Dancer::Route::Registry->hook('before', @_) }
+sub before_template { Dancer::Route::Registry->hook('before_template', @_) }
+sub captures     { Dancer::SharedData->request->params->{captures} }
 sub cookies      { Dancer::Cookies->cookies }
 sub config       { Dancer::Config::settings() }
 sub content_type { Dancer::Response::content_type(@_) }
@@ -112,8 +114,8 @@ sub from_yaml    { Dancer::Serializer::YAML::from_yaml(@_) }
 sub from_xml     { Dancer::Serializer::XML::from_xml(@_) }
 
 sub get {
-    Dancer::Route->add('head', @_);
-    Dancer::Route->add('get',  @_);
+    Dancer::App->current->registry->universal_add('head', @_);
+    Dancer::App->current->registry->universal_add('get', @_);
 }
 sub halt       { Dancer::Response->halt(@_) }
 sub headers    { Dancer::Response::headers(@_); }
@@ -125,17 +127,25 @@ sub mime_type  { Dancer::Config::mime_types(@_) }
 sub params     { Dancer::SharedData->request->params(@_) }
 sub pass       { Dancer::Response->pass }
 sub path       { Dancer::FileUtils::path(@_) }
-sub post       { Dancer::Route->add('post', @_) }
-sub prefix     { Dancer::Route->prefix(@_) }
-sub del        { Dancer::Route->add('delete', @_) }
-sub options    { Dancer::Route->add('options', @_) }
-sub put        { Dancer::Route->add('put', @_) }
+sub post       { Dancer::App->current->registry->universal_add('post', @_) }
+sub prefix     { Dancer::App->current->set_prefix(@_) }
+sub del        { Dancer::App->current->registry->universal_add('delete', @_) }
+sub options    { Dancer::App->current->registry->universal_add('options', @_) }
+sub put        { Dancer::App->current->registry->universal_add('put', @_) }
 sub r          { warn "'r' is DEPRECATED use qr{} instead" ; return {regexp => $_[0]} }
 sub redirect   { Dancer::Helpers::redirect(@_) }
 sub request    { Dancer::SharedData->request }
 sub send_file  { Dancer::Helpers::send_file(@_) }
-sub set        { Dancer::Config::setting(@_) }
-sub setting    { Dancer::Config::setting(@_) }
+sub set        { goto &setting }
+sub setting    { 
+    if (Dancer::App->applications) {
+        return Dancer::App->current->setting(@_);
+    }
+    else {
+        return Dancer::Config::setting(@_)
+    }
+}
+
 sub set_cookie { Dancer::Helpers::set_cookie(@_) }
 
 sub session {
@@ -162,16 +172,25 @@ sub var      { Dancer::SharedData->var(@_) }
 sub vars     { Dancer::SharedData->vars }
 sub warning  { goto &Dancer::Logger::warning }
 
+# FIXME handle previous usage of load_app with multiple app names
 sub load_app {
-    for my $app (@_) {
-        Dancer::Logger::core("loading application $app");
+    my ($app_name, %options) = @_;
+    Dancer::Logger::core("loading application $app_name");
+    
+    # set the application
+    my $app = Dancer::App->set_running_app($app_name);
+    
+    # Application options
+    $app->prefix($options{prefix}) if $options{prefix};
+    $app->settings($options{settings}) if $options{settings};
 
-        use lib path( dirname( File::Spec->rel2abs($0) ), 'lib' );
+    # load the application
+    use lib path( dirname( File::Spec->rel2abs($0) ), 'lib' );
+    eval "use $app_name";
+    die "unable to load application $app_name : $@" if $@;
 
-        # we want to propagate loading errors, so don't use ModuleLoader here
-        eval "use $app";
-        die "unable to load application $app : $@" if $@;
-    }
+    # restore the main application
+    Dancer::App->set_running_app('main');
 }
 
 sub load_plugin {
@@ -218,7 +237,6 @@ sub _init {
     setting logger  => 'file';
     setting confdir => $ENV{DANCER_CONFDIR} || setting('appdir');
     Dancer::Config->load;
-    Dancer::Route->init;
 }
 
 1;
@@ -281,20 +299,21 @@ involving Dancer and Plack, see L<Dancer::Deployment>.
 
 =head1 METHODS
 
-=head2 ajax
+=head2 after
 
-Define a route for 'ajax' query. To be matched, the request must have the B<X_REQUESTED_WITH> header set to B<XMLHttpRequet>.
+Add a hook at the B<after> position:
 
-    ajax '/list' => sub {
-       my $result = [qw/one two three/];
-       to_json($result);
-    }
+    after sub {
+        my $response = shift;
+        # do something with request
+    };
 
-or
+The anonymous function which is given to C<after> will be executed after
+having executed a route.
 
-    ajax ['get', 'post'] => '/list' => sub {
-        # code
-    }
+You can define multiple after filters, using the C<after> helper as
+many times as you wish; each filter will be executed, in the order you added
+them.
 
 =head2 any
 
@@ -325,6 +344,18 @@ You can define multiple before filters, using the C<before> helper as
 many times as you wish; each filter will be executed, in the order you added
 them.
 
+=head2 before_template
+
+Defines a before_template filter:
+
+    before_template sub {
+        # do something with request, vars or params
+    };
+
+The anonymous function which is given to C<before_template> will be executed
+before sending data and tokens to the template.
+
+This filter works as the C<before> and C<after> filter.
 
 =head2 cookies
 
@@ -722,7 +753,15 @@ Tells the route handler to build a response with the current template engine:
     };
 
 The first parameter should be a template available in the views directory, the
-second one (optional) is a hashref of tokens to interpolate.
+second one (optional) is a hashref of tokens to interpolate, and the third
+(again optional) is a hashref of options.
+
+For example, to disable the layout for a specific request:
+
+    get '/' => sub {
+        template 'index.tt', {}, { layout => undef };
+    };
+
 
 =head2 to_dumper
 
@@ -792,7 +831,9 @@ Returns a fully-qualified URI for the given path:
 
 =head2 captures
 
-If there are named captures in the route Regexp, captures returns a reference to a copy of %+
+If there are named captures in the route Regexp, captures returns a reference to a copy of %+.
+
+Named captures are a feature of Perl 5.10, and are not supported in earlier versions.
 
     get qr{
 	/ (?<object> user   | ticket | comment )
