@@ -35,6 +35,7 @@ my $setters = {
     },
     route_cache => sub {
         my ($setting, $value) = @_;
+        require Dancer::Route::Cache;
         Dancer::Route::Cache->reset();
     },
     serializer => sub {
@@ -46,6 +47,19 @@ my $setters = {
         my ($setting, $value) = @_;
         $^W = $value ? 1 : 0;
     },
+    auto_page => sub {
+        my ($setting, $auto_page) = @_;
+        if ($auto_page) {
+            require Dancer::App;
+            Dancer::App->current->registry->universal_add(
+                'get', '/:page',
+                sub {
+                    my $params = Dancer::SharedData->request->params;
+                    Dancer::Helpers::template($params->{'page'});
+                }
+            );
+        }
+    },
 };
 
 # public accessor for get/set
@@ -125,7 +139,7 @@ sub load_default_settings {
     $SETTINGS->{charset}      ||= $ENV{DANCER_CHARSET}      || '';
     $SETTINGS->{access_log}   ||= $ENV{DANCER_ACCESS_LOG}   || 1;
     $SETTINGS->{daemon}       ||= $ENV{DANCER_DAEMON}       || 0;
-    $SETTINGS->{apphandler}   ||= $ENV{DANCER_APPHANDLER}   || 'standalone';
+    $SETTINGS->{apphandler}   ||= $ENV{DANCER_APPHANDLER}   || 'Standalone';
     $SETTINGS->{warnings}     ||= $ENV{DANCER_WARNINGS}     || 0;
     $SETTINGS->{auto_reload}  ||= $ENV{DANCER_AUTO_RELOAD}  || 0;
     $SETTINGS->{environment}
@@ -146,7 +160,7 @@ __END__
 
 =head1 NAME
 
-Dancer::Config - Setting registry for Dancer
+Dancer::Config - setting registry for Dancer
 
 =head1 DESCRIPTION
 
