@@ -9,8 +9,21 @@ use base 'Dancer::Handler', 'HTTP::Server::Simple::PSGI';
 use Dancer::HTTP;
 use Dancer::GetOpt;
 use Dancer::Config 'setting';
+use Dancer::Headers;
 use Dancer::FileUtils qw(read_glob_content);
+use Dancer::SharedData;
 
+# we have to subclass headers, from HTTP::Server::Simple in order to catch them
+sub headers {
+    my ($self, $headers) = @_;
+    
+    # first, save the headers
+    my $dh = Dancer::Headers->new(headers => $headers);
+    Dancer::SharedData->headers($dh);
+    
+    # then, let the SUPER method do its job
+    $self->SUPER::headers($headers);
+}
 
 # in standalone mode, this method initializes the process
 # and start an HTTP server
@@ -26,7 +39,6 @@ sub start {
         $dancer->handle_request($req);
     };
 
-    Dancer::Route->init();
     $dancer->app($app);
 
     if (setting('daemon')) {
