@@ -31,6 +31,7 @@ sub send_file {
 
 sub template {
     my ($view, $tokens, $options) = @_;
+
     $options ||= {layout => 1};
     my $layout = setting('layout');
     undef $layout unless $options->{layout};
@@ -43,6 +44,7 @@ sub template {
     }
 
     $view = Dancer::Template->engine->view($view);
+
     if (!-r $view) {
         my $error = Dancer::Error->new(
             code    => 404,
@@ -51,6 +53,9 @@ sub template {
         return Dancer::Response::set($error->render);
     }
 
+    my $app = Dancer::App->current;
+    $_->($tokens) for ( @{ $app->registry->hooks->{before_template} } );
+
     my $content = Dancer::Template->engine->render($view, $tokens);
     return $content if not defined $layout;
 
