@@ -8,6 +8,7 @@ use base 'Exporter';
 use vars qw(@EXPORT);
 
 @EXPORT = qw(
+    add_hook
     register
     register_plugin
     plugin_setting
@@ -15,7 +16,9 @@ use vars qw(@EXPORT);
 
 my @_reserved_keywords = @Dancer::EXPORT;
 
-my $_keywords = [];
+my $_keywords = {};
+
+sub add_hook { Dancer::Route::Registry->hook(@_) }
 
 sub plugin_setting {
     my $plugin_orig_name = caller();
@@ -23,28 +26,48 @@ sub plugin_setting {
     
     my $settings = setting('plugins');
 
-    foreach ($plugin_name, $plugin_orig_name) {
+    foreach (   $plugin_name,    $plugin_orig_name,
+             lc $plugin_name, lc $plugin_orig_name) 
+    {
         return $settings->{$_}
             if ( exists $settings->{$_} );
     }
     return undef;
 }
 
-
 sub register($$) {
     my ($keyword, $code) = @_;
+    my $plugin_name = caller();
+
     if (grep {$_ eq $keyword} @_reserved_keywords) {
         die "You can't use $keyword, this is a reserved keyword";
     }
-    push @$_keywords, {$keyword => $code};
+    $_keywords->{$plugin_name} ||= [];
+    push @{$_keywords->{$plugin_name}}, [$keyword => $code];
 }
 
 sub register_plugin {
+    my ($application) = shift || caller(1);
     my ($plugin) = caller();
-    my ($application) = caller(1);
 
-    for my $keyword (@$_keywords) {
-        my ($name, $code) = each (%$keyword);
+    export_plugin_symbols($plugin => $application); 
+}
+
+sub load_plugin {
+    my ($plugin) = @_;
+    my $application = caller();
+
+    eval "use $plugin";
+    die "unable to load plugin '$plugin' : $@" if $@;
+
+    export_plugin_symbols($plugin => $application);
+}
+
+sub export_plugin_symbols {
+    my ($plugin, $application) = @_;
+
+    for my $keyword (@{ $_keywords->{$plugin} }) {
+        my ($name, $code) = @$keyword;
         {
             no strict 'refs';
             *{"${application}::${name}"} = $code;
@@ -59,7 +82,7 @@ __END__
 
 =head1 NAME
 
-Dancer::Plugin
+Dancer::Plugin - helper for writing Dancer plugins
 
 =head1 DESCRIPTION
 
