@@ -32,6 +32,7 @@ Dancer::Request->attributes(
 sub agent                 { $_[0]->user_agent }
 sub remote_address        { $_[0]->{env}->{'REMOTE_ADDR'} }
 sub forwarded_for_address { $_[0]->{env}->{'X_FORWARDED_FOR'} }
+sub is_head               { $_[0]->{method} eq 'HEAD' }
 sub is_post               { $_[0]->{method} eq 'POST' }
 sub is_get                { $_[0]->{method} eq 'GET' }
 sub is_put                { $_[0]->{method} eq 'PUT' }
@@ -70,6 +71,11 @@ sub new {
     return $self;
 }
 
+sub to_string {
+    my ($self) = @_;
+    return "[#".$self->id."] ".$self->method." ".$self->path;
+}
+
 # helper for building a request object by hand
 # with forced method, path and params.
 sub new_for_request {
@@ -460,16 +466,35 @@ would return C<http://localhost:5000/foo/bar?baz=baz>.
 
 =head2 params($source)
 
-If no source given, return a mixed hashref containing all the parameters that
-have been parsed.
-Be aware it's a mixed structure, so if you use multiple
-variables with the same name in your route pattern, query string or request
-body, you can't know for sure which value you'll get there.
+Called in scalar context, returns a hashref of params, either from the specified
+source (see below for more info on that) or merging all sources.
 
-If you need to use the same name for different sources of input, use the
-C<$source> option, like the following:
+So, you can use, for instance:
 
-If source equals C<route>, then only params parsed from route pattern
+    my $foo = params->{foo}
+
+If called in list context, returns a list of key => value pairs, so you could use:
+
+    my %allparams = params;
+
+
+=head3 Fetching only params from a given source
+
+If a required source isn't specified, a mixed hashref (or list of key value
+pairs, in list context) will be returned; this will contain params from all
+sources (route, query, body).
+
+In practical terms, this means that if the param C<foo> is passed both on the
+querystring and in a POST body, you can only access one of them.
+
+If you want to see only params from a given source, you can say so by passing
+the C<$source> param to C<params()>:
+
+    my %querystring_params = params('query');
+    my %route_params       = params('route');
+    my %post_params        = params('body');
+
+If source equals C<route>, then only params parsed from the route pattern
 are returned.
 
 If source equals C<query>, then only params parsed from the query string are
