@@ -6,6 +6,7 @@ use warnings;
 use Test::More import => ['!pass'];
 
 use Dancer ':syntax';
+use Dancer::App;
 use Dancer::Request;
 use Dancer::SharedData;
 use Dancer::Renderer;
@@ -19,19 +20,30 @@ use vars '@EXPORT';
 
     response_exists
     response_doesnt_exist
+
     response_status_is
     response_status_isnt
+    
     response_content_is
     response_content_isnt
     response_content_is_deeply
     response_content_like
     response_content_unlike
-
     response_is_file
+
+    response_headers_are_deeply
+    get_response
 );
 
 sub import {
     my ($class, %options) = @_;
+
+    # mimic PSGI env
+    $ENV{SERVERNAME} = 'localhost';
+    $ENV{HTTP_HOST} = 'localhost';
+    $ENV{SERVER_PORT} = 80;
+    $ENV{'psgi.url_scheme'} = 'http';
+
     my ($package, $script) = caller;
     $class->export_to_level(1, $class, @EXPORT );
 
@@ -48,7 +60,7 @@ sub route_exists {
     $test_name ||= "a route exists for $method $path";
     
     $req = Dancer::Request->new_for_request($method => $path);
-    ok(Dancer::Route->find($path, $method, $req), $test_name);
+    ok(Dancer::App->find_route_through_apps($req), $test_name);
 }
 
 sub route_doesnt_exist {
@@ -58,7 +70,7 @@ sub route_doesnt_exist {
     $test_name ||= "no route exists for $method $path";
 
     $req = Dancer::Request->new_for_request($method => $path);
-    ok((!defined(Dancer::Route->find($path, $method, $req))), $test_name);
+    ok(! defined(Dancer::App->find_route_through_apps($req)), $test_name);
 }
 
 # Response status
@@ -67,7 +79,7 @@ sub response_exists {
     my ($req, $test_name) = @_;
     $test_name ||= "a response is found for @$req";
 
-    my $response = _get_response($req);
+    my $response = get_response($req);
     ok(defined($response), $test_name);
 }
 
@@ -75,7 +87,7 @@ sub response_doesnt_exist {
     my ($req, $test_name) = @_;
     $test_name ||= "no response found for @$req";
 
-    my $response = _get_response($req);
+    my $response = get_response($req);
     ok(!defined($response), $test_name);
 }
 
@@ -83,7 +95,7 @@ sub response_status_is {
     my ($req, $status, $test_name) = @_;
     $test_name ||= "response status is $status for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     is $response->{status}, $status, $test_name;
 }
 
@@ -91,7 +103,7 @@ sub response_status_isnt {
     my ($req, $status, $test_name) = @_;
     $test_name ||= "response status is not $status for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     isnt $response->{status}, $status, $test_name;
 }
 
@@ -101,7 +113,7 @@ sub response_content_is {
     my ($req, $matcher, $test_name) = @_;
     $test_name ||= "response content looks good for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     is $response->{content}, $matcher, $test_name;
 }
 
@@ -109,7 +121,7 @@ sub response_content_isnt {
     my ($req, $matcher, $test_name) = @_;
     $test_name ||= "response content looks good for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     isnt $response->{content}, $matcher, $test_name;
 }
 
@@ -117,7 +129,7 @@ sub response_content_like {
     my ($req, $matcher, $test_name) = @_;
     $test_name ||= "response content looks good for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     like $response->{content}, $matcher, $test_name;
 }
 
@@ -125,7 +137,7 @@ sub response_content_unlike {
     my ($req, $matcher, $test_name) = @_;
     $test_name ||= "response content looks good for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     unlike $response->{content}, $matcher, $test_name;
 }
 
@@ -133,7 +145,7 @@ sub response_content_is_deeply {
     my ($req, $matcher, $test_name) = @_;
     $test_name ||= "response content looks good for @$req";
 
-    my $response = _get_response($req); 
+    my $response = get_response($req); 
     is_deeply $response->{content}, $matcher, $test_name;
 }
 
@@ -145,7 +157,17 @@ sub response_is_file {
     ok(defined($response), $test_name);
 }
 
-sub _get_response {
+sub response_headers_are_deeply {
+    my ($req, $expected, $test_name) = @_;
+    $test_name ||= "headers are as expected for @$req";
+
+    my $response = get_response($req);
+    is_deeply($response->{headers}, $expected, $test_name);
+}
+
+# private
+
+sub get_response {
     my ($req) = @_;
     my ($method, $path, $params, $headers) = @$req;
     my $request = Dancer::Request->new_for_request($method => $path, $params);
@@ -303,6 +325,31 @@ given.
     response_content_unlike [GET => '/'], qr/Page not found/, 
         "response content looks good for GET /";
 
+=head2 response_headers_are_deeply([$method, $path], $expected, $test_name)
+
+Asserts that the response headers data structure equals the one given.
+
+    response_headers_are_deeply [GET => '/'], [ 'X-Powered-By' => 'Dancer 1.150' ];
+
+=head2 get_response([$method, $path])
+
+Returns a Dancer::Response object for the given request. The status and content
+can both be accessed with this object. A good reason to use this function is for
+testing POST requests. Since POST requests may not be idempotent, it is
+necessary to capture the content and status in one shot. Calling the
+response_status_is and response_content_is functions in succession would make
+two requests, each of which could alter the state of the application and cause
+Schrodinger's cat to die.
+
+    my $response = get_response [POST => '/widgets'];
+    is $response->{status}, 202, "response for POST /widgets is 202";
+    is $response->{content}, "Widget #1 has been scheduled for creation",
+        "response content looks good for first POST /widgets";
+
+    $response = get_response [POST => '/widgets'];
+    is $response->{status}, 202, "response for POST /widgets is 202";
+    is $response->{content}, "Widget #2 has been scheduled for creation",
+        "response content looks good for second POST /widgets";
 
 =head1 LICENSE
 
